clear all
close all
disp(' ')
disp('*******************************************')
disp('* Pathfinding with Comsol 5.0 - Example 1 *')
disp('*******************************************')
atot=tic;

tic
%%% Initializing Comsol
disp('Comsol initialisation:')
import com.comsol.model.*
import com.comsol.model.util.*
model = ModelUtil.create('Model');
toc

tic
disp('Building geometry:')
model.geom.label(['Subsecuencias de la geometr' native2unicode(hex2dec({'00' 'ed'}), 'unicode') 'a']);
model.geom.create('geom1', 2);

model.mesh.create('mesh1', 'geom1');

model.geom('geom1').label(['Geometr' native2unicode(hex2dec({'00' 'ed'}), 'unicode') 'a 1']);
model.geom('geom1').create('imp1', 'Import');
model.geom('geom1').feature('imp1').label('Importar 1');
model.geom('geom1').feature('imp1').set('type', 'dxf');
model.geom('geom1').feature('imp1').set('filename', 'labyrinthe.dxf');
model.geom('geom1').feature('imp1').importData;
model.geom('geom1').create('sca1', 'Scale');
model.geom('geom1').feature('sca1').set('anisotropic', {'1.0219568254806' '1.0303970246542'});
model.geom('geom1').feature('sca1').set('pos', {'76.081425899213' '58.252848237644'});
model.geom('geom1').feature('sca1').set('type', 'anisotropic');
model.geom('geom1').feature('sca1').set('factor', {'1.0219568254806' '1.0303970246542'});
model.geom('geom1').feature('sca1').selection('input').set({'imp1'});
model.geom('geom1').create('b1', 'BezierPolygon');
model.geom('geom1').feature('b1').label(['Pol' native2unicode(hex2dec({'00' 'ed'}), 'unicode') 'gono de B' native2unicode(hex2dec({'00' 'e9'}), 'unicode') 'zier 1']);
model.geom('geom1').feature('b1').set('degree', {'1' '1' '1' '1'});
model.geom('geom1').feature('b1').set('p', {'0' '130' '130' '0' '0'; '110' '110' '0' '0' '110'});
model.geom('geom1').feature('b1').set('w', {'1' '1' '1' '1' '1' '1' '1' '1'});
model.geom('geom1').create('dif1', 'Difference');
model.geom('geom1').feature('dif1').label('Diferencia 1');
model.geom('geom1').feature('dif1').selection('input').set({'b1'});
model.geom('geom1').feature('dif1').selection('input2').set({'sca1'});
model.geom('geom1').create('e1', 'Ellipse');
model.geom('geom1').feature('e1').label('Elipse 1');
model.geom('geom1').feature('e1').set('pos', {'8' '108'});
model.geom('geom1').feature('e1').set('semiaxes', {'1.5' '1.5'});
model.geom('geom1').create('e2', 'Ellipse');
model.geom('geom1').feature('e2').label('Elipse 2');
model.geom('geom1').feature('e2').set('pos', {'127' '9'});
model.geom('geom1').feature('e2').set('semiaxes', {'1' '1'});
model.geom('geom1').create('dif5', 'Difference');
model.geom('geom1').feature('dif5').label('Diferencia 5');
model.geom('geom1').feature('dif5').selection('input').set({'dif1'});
model.geom('geom1').feature('dif5').selection('input2').set({'e1' 'e2'});
model.geom('geom1').create('uni1', 'Union');
model.geom('geom1').feature('uni1').label(['Uni' native2unicode(hex2dec({'00' 'f3'}), 'unicode') 'n 1']);
model.geom('geom1').feature('uni1').selection('input').set({'dif5'});
model.geom('geom1').feature('fin').label(['Formar uni' native2unicode(hex2dec({'00' 'f3'}), 'unicode') 'n']);
model.geom('geom1').run;
toc

tic
disp('Mesh creation:')
model.mesh('mesh1').create('size1', 'Size');
model.mesh('mesh1').create('cr1', 'CornerRefinement');
model.mesh('mesh1').create('ftri1', 'FreeTri');
model.mesh('mesh1').feature('size1').set('hauto', 2.0);
model.mesh('mesh1').run;
toc


tic
disp('Model creation and resolution:')
model.physics.create('spf2', 'LaminarFlow', 'geom1');
model.physics('spf2').identifier('spf2');
model.physics('spf2').field('velocity').field('u2');
model.physics('spf2').field('pressure').field('p2');
model.physics('spf2').field('turbulentkineticenergy').field('k2');
model.physics('spf2').field('turbulentdissipationrate').field('ep2');
model.physics('spf2').field('specificdissipationrate').field('om2');
model.physics('spf2').field('reciprocallength').field('G2');
model.physics('spf2').field('correctedvelocity').field('uc2');
model.physics('spf2').field('correctedvelocity').component({'uc2x' 'uc2y' 'uc2z'});
model.physics('spf2').field('correctedpressure').field('pc2');
model.physics('spf2').field('turbulentkinematicviscosity').field('nutilde2');
model.physics('spf2').field('dimensionless1').field('yPlus2');
model.physics('spf2').field('dimensionless2').field('uPlus2');
model.physics('spf2').create('inl1', 'InletBoundary', 1);
model.physics('spf2').feature('inl1').selection.set([164 165 166 167]);
model.physics('spf2').create('out1', 'OutletBoundary', 1);
model.physics('spf2').feature('out1').selection.set([168 169 170 171]);
model.physics('spf2').prop('PhysicalModelProperty').set('Compressibility', 'Incompressible');
model.physics('spf2').prop('AdvancedSettingProperty').set('locCFL', 'spf2.localCFLvalue');
model.physics('spf2').feature('fp1').set('rho_mat', 'userdef');
model.physics('spf2').feature('fp1').set('rho', '1010');
model.physics('spf2').feature('fp1').set('mu_mat', 'userdef');
model.physics('spf2').feature('fp1').set('mu', '9.8*10^(-4)');
model.physics('spf2').feature('fp1').label('Propiedades de fluidos 1');
model.physics('spf2').feature('inl1').set('U0in', '1e-7');

model.study.create('std1');
model.study('std1').create('stat', 'Stationary');
model.study('std1').feature('stat').set('initstudyhide', 'on');
model.study('std1').feature('stat').set('initsolhide', 'on');
model.study('std1').feature('stat').set('solnumhide', 'on');
model.study('std1').feature('stat').set('notstudyhide', 'on');
model.study('std1').feature('stat').set('notsolhide', 'on');
model.study('std1').feature('stat').set('notsolnumhide', 'on');

model.sol.create('sol1');
model.sol('sol1').study('std1');
model.sol('sol1').attach('std1');
model.sol('sol1').create('st1', 'StudyStep');
model.sol('sol1').create('v1', 'Variables');
model.sol('sol1').create('s1', 'Stationary');
model.sol('sol1').feature('s1').create('fc1', 'FullyCoupled');
model.sol('sol1').feature('s1').create('d1', 'Direct');
model.sol('sol1').feature('s1').feature.remove('fcDef');
model.sol('sol1').attach('std1');
model.sol('sol1').feature('s1').feature('fc1').set('initstep', '0.01');
model.sol('sol1').feature('s1').feature('fc1').set('minstep', '1.0E-6');
model.sol('sol1').feature('s1').feature('d1').set('linsolver', 'pardiso');
model.sol('sol1').runAll;
toc

tic
disp('Creating Outputs:')
model.result.create('pg1', 'PlotGroup2D');
model.result('pg1').create('str4', 'Streamline');
model.result('pg1').feature('str4').selection.set([164 165 166 167]);
model.result.export.create('plot2', 'Plot');
model.result('pg1').feature('str4').set('color', 'blue');
model.result('pg1').feature('str4').set('selnumber', '50');
model.result.export.label('Exportar');
model.result.export('plot2').label('All sstream');
model.result.export('plot2').set('filename', 'allsl.txt');
model.result.export('plot2').set('plot', 'str4');
model.result.report.label('Reportes');
model.result.export('plot2').run


aa=load('allsl.txt');
longm=inf;
bb=sort(unique(aa(:,3)));

for ii=1:length(bb)
   uu=aa((aa(:,3)==bb(ii)),1); 
   vv=aa((aa(:,3)==bb(ii)),2); 
   if (uu(end)>120)&(vv(end)<20)
       lengv=0;
       for jj=1:length(uu)-1
       lengv=lengv+sqrt((uu(jj)-uu(jj+1))^2+(vv(jj)-vv(jj+1))^2);
       end
   if lengv<longm
       longm=lengv;
       uum=uu;vvm=vv;
   end
   end 
end

toc
disp(['Total time: ' num2str(toc(atot)) ' (s)'])

vsl=0;
figure(2)
clf
axes('fontsize',14)
hold on
mphgeom(model)
for ii=1:length(bb)
   uu=aa((aa(:,3)==bb(ii)),1); 
   vv=aa((aa(:,3)==bb(ii)),2); 
   if (uu(end)>120)&(vv(end)<10)
      lengv(ii)=0;
       for jj=1:length(uu)-1
       lengv(ii)=lengv(ii)+sqrt((uu(jj)-uu(jj+1))^2+(vv(jj)-vv(jj+1))^2);
       end
       vsl=vsl+1;
   plot(uu,vv,'color',[0.7,0.7,0.7],'linewidth',1)
   end 
end
plot(uum,vvm,'k','linewidth',3)
title('Example 1')
axis tight

